package com.qiyuesuo.sdk.api;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;

import com.qiyuesuo.sdk.contract.Contract;
import com.qiyuesuo.sdk.exception.AuthenticationException;
import com.qiyuesuo.sdk.exception.BaseSdkException;
import com.qiyuesuo.sdk.exception.HttpException;
import com.qiyuesuo.sdk.exception.ServiceException;
import com.qiyuesuo.sdk.sign.SignUrlRequest;
import com.qiyuesuo.sdk.sign.SignUrlResponse;
import com.qiyuesuo.sdk.sign.Stamper;
import com.qiyuesuo.sdk.sign.ViewUrlResponse;
import com.qiyuesuo.sdk.signer.Signer;
import com.qiyuesuo.sdk.template.Template;

/**
 * 远程签服务接口
 * 远程签需要将合同文件保存在契约锁服务器
 */
public interface RemoteSignService {

	/**
	 * 根据文件创建合同,默认30天后过期
	 * @param input 文件输入流
	 * @param docName 合同文档名称
	 * @return 服务端生成的合同文档ID
	 * @throws BaseSdkException
	 */
	Long create(InputStream input, String docName) throws BaseSdkException;
	
	/**
	 * 根据文件创建合同，指定合同到期日期
	 * @param input 输入合同文件流
	 * @param docName 合同文档名称
	 * @param expireTime 过期时间
	 * @return 服务端生成的合同文档ID
	 * @throws BaseSdkException SDK异常
	 */
	Long create(InputStream input, String docName, Date expireTime) throws BaseSdkException;

	/**
	 * 根据模板创建合同,默认30天后过期
	 * @param template 模板对象
	 * @param docName 合同文档名称
	 * @return 服务端生成的合同文档ID
	 * @throws BaseSdkException SDK异常
	 */
	Long create(Template template,String docName) throws BaseSdkException;
	
	/**
	 * 根据模板创建合同
	 * @param template 模板对象
	 * @param docName 合同文档名称
	 * @param expireTime 过期时间
	 * @return 服务端生成的合同文档ID
	 * @throws BaseSdkException SDK异常
	 */
	Long create(Template template,String docName, Date expireTime) throws BaseSdkException;

	/**
	 * 根据html创建合同,默认30天后过期
	 * @param html html格式的合同文档
	 * @param docName 合同文档名称
	 * @return 服务端生成的合同文档ID
	 * @throws BaseSdkException SDK异常
	 */
	Long create(String html, String docName) throws BaseSdkException;
	
	/**
	 * 根据html创建合同
	 * @param html html格式的合同文档
	 * @param docName 合同文档名称
	 * @param expireTime 过期时间
	 * @return 服务端生成的合同文档ID
	 * @throws BaseSdkException SDK异常
	 */
	Long create(String html, String docName, Date expireTime) throws BaseSdkException;

	/**
	 * 平台签署(无签名外观)
	 * 
	 * @param documentId 文档ID
	 * @throws BaseSdkException SDK异常
	 */
	void sign(Long documentId) throws BaseSdkException;
	/**
	 * 平台签署(有签名外观)
	 * 
	 * @param documentId 文档ID
	 * @param sealId 印章ID
	 * @param stamper 签名位置
	 * @throws BaseSdkException
	 */
	void sign(Long documentId,Long sealId,Stamper stamper) throws BaseSdkException;

	/**
	 * 用户签署（无签名外观）
	 * 
	 * @param documentId 文档ID
	 * @param signer 签署人
	 * @throws BaseSdkException SDK异常
	 */
	void sign(Long documentId,Signer signer) throws BaseSdkException;
	/**
	 * 用户签署（有签名外观）
	 * 
	 * @param documentId 文档ID
	 * @param signer 签署人
	 * @param sealData 签名数据
	 * @param stamper 签名位置
	 * @throws BaseSdkException SDK异常
	 */
	void sign(Long documentId,Signer signer,String sealData,Stamper stamper) throws BaseSdkException;

	/**
	 * 获取用户签署的页面链接，用于签署可见签名
	 * @param request
	 * @return
	 * @throws BaseSdkException
	 */
	SignUrlResponse signUrl(SignUrlRequest request) throws BaseSdkException;
	
	/**
	 * 完成合同签署，并进行封存
	 * 
	 * @param documentId				合同文档ID
	 * @throws BaseSdkException			SDK异常
	 * @throws HttpException			服务错误
	 * @throws AuthenticationException	认证错误
	 * @throws ServiceException			远程服务错误
	 */
	void complete(Long documentId) throws BaseSdkException;

	/**
	 * 下载合同文档（PDF格式）
	 * @param documentId	合同文档ID
	 * @param doc	合同文件输出数据流（PDF格式）
	 * @throws BaseSdkException
	 */
	void download(Long documentId,OutputStream doc) throws BaseSdkException;
	
	/**
	 * 获取合同详情
	 * @param documentId 合同文档ID
	 * @throws BaseSdkException
	 */
	Contract detail(Long documentId)  throws BaseSdkException;
	
	/**
	 * 获取查看合同页面URL
	 * @param documentId 文档ID
	 * @return 浏览合同URL
	 * @throws BaseSdkException
	 */
	public ViewUrlResponse viewUrl(Long documentId) throws BaseSdkException;
	
}
